"""Provides the application configuration."""

from fastapi_factory_utilities.core.app import AppConfigAbstract


class AppConfig(AppConfigAbstract):
    """The application configuration.

    Values are set in application.yaml.
    """

    pass
